import json
import os
import platform
import random
import re
import string
import urllib

import yaml
from requests import Response

from autotests_coverage.config import EnvConfig
from autotests_coverage.uri import URI


class ApiDocsManagerBase:
    def __init__(self, uri: URI, response: Response, kwargs: dict, method: str = None):
        self.variables = EnvConfig.get_variables()

        self._uri = uri
        self._method = method
        self._response: Response = response
        self._other_request_params = kwargs

    def _get_path_params(self) -> list:
        params_ = []

        for key, value in self._uri.uri_params.items():
            params_.append(
                {
                    "name": key,
                    "in": "path",
                    "required": False,
                    "x-example": urllib.parse.unquote(str(value)),
                }
            )

        return params_

    def _get_body_params(self):
        try:
            request_body = json.loads(self._response.request.body)
        except Exception:
            request_body = None

        if request_body:
            types = {
                "object": "object",
                "str": "string",
                "int": "number",
                "float": "number",
                "bool": "boolean",
                "list": "array",
            }

            if isinstance(request_body, dict):
                properties = {}

                for k, v in request_body.items():
                    value_type = types.get(type(v).__name__, "object")

                    if value_type == "string":
                        value = urllib.parse.unquote(str(v))
                    else:
                        value = v

                    properties[k] = {k: value, "type": value_type}

                request_body: dict = {
                    "content": {
                        "application/json": {
                            "schema": {"type": "object", "properties": properties},
                            "example": json.loads(self._response.request.body),
                        }
                    }
                }
            elif isinstance(request_body, list):
                items_type = types.get(type(request_body[0]).__name__, "object")
                request_body: dict = {
                    "content": {
                        "application/json": {
                            "schema": {
                                "type": "array",
                                "items": {"type": items_type},
                            },
                            "example": json.loads(self._response.request.body),
                        }
                    }
                }
            else:
                request_body: dict = {
                    "content": {
                        "application/json": {
                            "schema": {
                                "type": "string",
                            },
                            "example": urllib.parse.unquote(
                                str(self._response.request.body)
                            ),
                        }
                    }
                }
        else:
            request_body = None

        return request_body

    def _get_other_request_params(self, params_key: str, params_in: str) -> list:
        prams_raw = self._other_request_params.get(params_key, {})

        if prams_raw:
            params = list(prams_raw.items())
        else:
            params = []

        raw = self._uri.raw.split("?")

        if len(raw) > 1:
            params += [tuple(x.split("=")) for x in str(raw[1]).split("&")]

        if not params:
            return []

        params_ = []

        for key, value in params:
            params_.append(
                {
                    "name": key,
                    "in": params_in,
                    "required": False,
                    "x-example": urllib.parse.unquote(str(value)),
                }
            )

        return params_

    def _get_query_params(self) -> list:
        return self._get_other_request_params(params_key="params", params_in="query")

    def _get_header_params(self) -> list:
        return self._get_other_request_params(params_key="headers", params_in="header")

    def __get_output_subdir(self):
        return re.match(r"(^\w*)://(.*)", self._uri.host).group(2)

    def __custom_pystr(self, min_chars=5, max_chars=5):
        length = random.randint(min_chars, max_chars) if min_chars != max_chars else min_chars

        characters = string.ascii_letters + string.digits
        random_string = ''.join(random.choice(characters) for _ in range(length))

        return random_string

    def write_schema(self):
        schema_dict = self._get_schema()

        rnd = self.__custom_pystr(min_chars=5, max_chars=5)
        file_name = f"{self._method.upper()} {self._uri.formatted[1::]}".replace(
            "/", "-"
        ).replace(":", "_")

        path_ = os.path.join(str(self.variables.coverage_reports_dir), self.__get_output_subdir())
        file_path = f"{path_}/{file_name}".split("?")[0]
        file_path = f"{file_path} ({rnd}).{self.variables.api_docs_format}"

        try:
            with open(file_path, "w+") as file:
                if self.variables.api_docs_format == "yaml":
                    file.write(yaml.safe_dump(schema_dict, indent=4, sort_keys=False))
                elif self.variables.api_docs_format == "json":
                    file.write(json.dumps(schema_dict, indent=4))
                else:
                    raise Exception(
                        f"Unexpected docs format: {self.variables.api_docs_format}. Valid formats: json, yaml"
                    )

        except FileNotFoundError as e:
            system_ = platform.system()
            abs_path = os.path.abspath(file_path)

            if system_ == "Windows" and len(abs_path) > 256:
                raise EnvironmentError(
                    f"Absolute path length is greater than 256 symbols:\n"
                    f"{abs_path}\n"
                    f"To remove this restriction you can use this guide: "
                    f"https://docs.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation#enable-long"
                    f"-paths-in-windows-10-version-1607-and-later "
                )
            else:
                raise Exception(
                    f"Cannot write to file.\n"
                    f"Path: {abs_path}\n"
                    f"Details: {e.strerror}"
                )

        return schema_dict



from requests import Response

from autotests_coverage.results_writers.base_schemas_manager import ApiDocsManagerBase
from autotests_coverage.uri import URI


class OpenApiSchemasManager(ApiDocsManagerBase):
    def __init__(self, uri: URI, method: str, response: Response, kwargs: dict):
        super().__init__(uri, response, kwargs, method)

    def _paths(self):
        path_ = self._uri.raw.split("?")[0]
        params = (
            self._get_path_params()
            + self._get_query_params()
            + self._get_header_params()
        )
        dict_ = {
            path_: {
                self._method: {
                    "parameters": params,
                    "responses": {self._response.status_code: {}},
                }
            }
        }

        body_params = self._get_body_params()

        if body_params:
            dict_[path_][self._method]["requestBody"] = body_params

        return dict_

    def _get_schema(self):
        schema_dict = {
            self.variables.api_docs_type: self.variables.api_docs_version,
            "info": {"title": "Recorded Request"},
            "paths": self._paths(),
        }

        return schema_dict



import os
from pathlib import Path

from pydantic import BaseModel, ValidationError, field_validator
from functools import lru_cache


class EnvConfig(BaseModel):
    tmp_configs_dir: str
    api_docs_type: str
    api_docs_version: str
    api_docs_format: str
    debug_mode: bool
    coverage_reports_dir: str
    coverage_configs_dir: str

    @classmethod
    @lru_cache(maxsize=None)
    def get_variables(cls) -> "EnvConfig":
        try:
            return cls(
                tmp_configs_dir=os.path.join(Path(__file__).parent, "tmp_configs_dir"),
                api_docs_type=os.environ.get("API_DOCS_TYPE", default="openapi"),
                api_docs_version=os.environ.get("API_DOCS_VERSION", default="3.0.0"),
                api_docs_format=os.environ.get("API_DOCS_FORMAT", default="json"),
                debug_mode=os.environ.get("DEBUG_MODE", default=False),
                coverage_reports_dir=os.environ.get("COVERAGE_REPORTS_DIR"),
                coverage_configs_dir=os.environ.get("COVERAGE_CONFIGS_DIR")
            )
        except ValidationError as e:
            raise ValueError(f"Ошибка валидации конфигурации: {e}")

    @field_validator("coverage_reports_dir", "coverage_configs_dir")
    def check_not_none(cls, value):
        if value is None:
            raise ValueError("Значение не может быть None")
        return value



import json
import os
import platform
import re
import shutil
import subprocess
from pathlib import Path
from typing import List

import requests
from filelock import FileLock

from autotests_coverage.config import EnvConfig
from autotests_coverage.docs_writers.api_doc_writer import write_api_doc_to_file


class CoverageReporter:
    def __init__(
        self,
        api_name: str,
        host: str,
        coverage_config_file_name: str,
        use_custom_config: bool = False,
        verify: bool = True
    ):
        self.variables = EnvConfig.get_variables()

        self.host = host
        self.verify = verify
        self.output_dir = self.__get_output_dir(host=self.host)
        self.swagger_doc_file = (f"{self.variables.coverage_reports_dir}/swagger-doc-{api_name}."
                                 f"{self.variables.api_docs_format}")

        if not use_custom_config:
            self.coverage_config_file_path = self.__copy_config_file_to_tmp_dir(
                file_path=os.path.join(self.variables.coverage_configs_dir, coverage_config_file_name),
                api_name=api_name
            )

        else:
            self.coverage_config_file_path = os.path.join(
                self.variables.coverage_configs_dir,
                coverage_config_file_name
            )

        self.ignored_paths = self.__get_ignored_paths_from_config()

    def __copy_config_file_to_tmp_dir(self, file_path: str, api_name: str):
        tmp_file_path = os.path.join(self.variables.tmp_configs_dir, f"swagger-coverage-config-{api_name}.json")

        lock_path = file_path + ".lock"
        lock = FileLock(lock_path)

        with lock:
            if not os.path.isfile(tmp_file_path):
                tmp_file_path = shutil.copy(
                    file_path, os.path.join(
                        self.variables.tmp_configs_dir, f"swagger-coverage-config-{api_name}.json"
                    )
                )

                with open(tmp_file_path, "r+", encoding="utf-8") as f:
                    file_data = json.load(f)

                file_data["writers"] = {
                    "html": {
                        "locale": "ru",
                        "filename": f"{self.variables.coverage_reports_dir}/{api_name}-coverage.html"
                    }
                }

                with open(tmp_file_path, 'w', encoding='utf-8') as f:
                    json.dump(file_data, f, indent=2, ensure_ascii=False)

            return tmp_file_path

    def __get_output_dir(self, host: str):
        subdir = re.match(r"(^\w*)://(.*)", host).group(2)

        output_dir = os.path.join(str(self.variables.coverage_reports_dir), subdir)

        if not os.path.exists(output_dir):
            Path(output_dir).mkdir(parents=True, exist_ok=True)

        return output_dir

    def __get_ignored_paths_from_config(self) -> List[str]:
        """
        Reads the swagger-coverage-config-<api_name>.json file and returns
        a list of endpoints/paths to exclude from the report
        """
        paths_to_ignore = []

        with open(self.coverage_config_file_path, "r") as file:
            data = json.load(file)
            paths = data.get("rules").get("paths", {})

            if paths.get("enable", False):
                paths_to_ignore = paths.get("ignore")

        return paths_to_ignore

    def setup(self, path_to_swagger_json: str, auth: object = None, cookies: dict = None):
        """
        Setup all required attributes to generate report

        :param path_to_swagger_json: The relative URL path to the swagger.json (example: "/docs/api")
        :param auth: Authentication object acceptable by "requests" library
        :param cookies: Cookies dictionary. (Usage example: set this to bypass Okta auth locally)

        """
        link_to_swagger_json = f"{self.host}{path_to_swagger_json}"

        response = requests.get(
            link_to_swagger_json, auth=auth, cookies=cookies, verify=self.verify
        )

        assert response.ok, (
            f"Swagger doc is not pulled. See details: "
            f"{response.status_code} {response.request.url}"
            f"{response.content}\n{response.content}"
        )

        write_api_doc_to_file(
            self.swagger_doc_file,
            api_doc_data=response,
            paths_to_delete=self.ignored_paths,
        )

    def generate_report(self):
        inner_location = "swagger-coverage-commandline/bin/swagger-coverage-commandline"

        cmd_path = os.path.join(os.path.dirname(__file__), inner_location)

        assert Path(cmd_path).exists(), f"No commandline tools is found in following locations:\n{cmd_path}\n"

        if platform.system() == "Windows":
            cmd_path_with_ext = f"{cmd_path}.bat"

            assert Path(cmd_path_with_ext).exists(), f"File not found: {cmd_path_with_ext}"

            command = [cmd_path_with_ext, "-s", self.swagger_doc_file, "-i", self.output_dir]

        else:
            command = [cmd_path, "-s", self.swagger_doc_file, "-i", self.output_dir]

        if self.coverage_config_file_path:
            command.extend(["-c", self.coverage_config_file_path])

        # Suppress all output if not in debug mode
        if not self.variables.debug_mode:
            with open(os.devnull, 'w') as devnull:
                subprocess.run(command, stdout=devnull, stderr=devnull)
        else:
            subprocess.run(command)

    @staticmethod
    def cleanup_input_files():
        variables = EnvConfig.get_variables()

        if os.path.exists(variables.coverage_reports_dir):
            for item in os.listdir(variables.coverage_reports_dir):
                item_path = os.path.join(variables.coverage_reports_dir, item)

                if os.path.isfile(item_path) or os.path.islink(item_path):
                    os.unlink(item_path)
                elif os.path.isdir(item_path):
                    shutil.rmtree(item_path, ignore_errors=True)

        else:
            Path(variables.coverage_reports_dir).mkdir(parents=True, exist_ok=True)

        shutil.rmtree(variables.tmp_configs_dir, ignore_errors=True)

        Path(variables.tmp_configs_dir).mkdir(parents=True, exist_ok=True)



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="autotests-coverage",
    version="1.1.0",
    author="Jamal Zeinalov",
    author_email="jamal.zeynalov@gmail.com",
    description='Python adapter for "swagger-coverage" tool',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JamalZeynalov/swagger-coverage-py",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "requests>=2.32.3",
        "setuptools>=70.0.0",
        "PyYAML>=6.0.2",
        "pydantic>=2.11.1",
        "filelock>=3.18.0"
    ],
    python_requires=">=3.7",
    include_package_data=True,
)



import json

import requests
import yaml

from autotests_coverage.config import EnvConfig


def __delete_ignored_paths_from_json(api_doc_data: requests.Response, paths_to_delete: list) -> dict:
    data: dict = api_doc_data.json()

    for path in paths_to_delete:
        if path in data["paths"]:
            del data["paths"][path]

    return data


def __delete_ignored_paths_from_yaml(api_doc_data: requests.Response, paths_to_delete: list) -> dict:
    data: dict = yaml.safe_load(str(api_doc_data.text))

    for path in paths_to_delete:
        if path in data["paths"]:
            del data["paths"][path]

    return data


def __write_api_doc_to_json(file_path: str, api_doc_data: requests.Response, paths_to_delete: list):
    variables = EnvConfig.get_variables()

    api_doc_data = __delete_ignored_paths_from_json(api_doc_data, paths_to_delete)

    if variables.api_docs_type == "swagger" and not api_doc_data.get("swagger", None):
        api_doc_data["swagger"] = "2.0"

    with open(file_path, "w+") as file:
        file.write(json.dumps(api_doc_data))


def __write_api_doc_to_yaml(file_path: str, api_doc_data: requests.Response, paths_to_delete: list):
    variables = EnvConfig.get_variables()

    data = __delete_ignored_paths_from_yaml(api_doc_data, paths_to_delete)

    if variables.api_docs_type == "swagger" and not data.get("swagger", None):
        data["swagger"] = "2.0"

    with open(file_path, "w+") as file:
        file.write(yaml.safe_dump(data, indent=4, sort_keys=False))


def write_api_doc_to_file(file_path: str, api_doc_data: requests.Response, paths_to_delete: list):
    variables = EnvConfig.get_variables()

    if variables.api_docs_format == "json":
        __write_api_doc_to_json(file_path, api_doc_data, paths_to_delete)
    else:
        __write_api_doc_to_yaml(file_path, api_doc_data, paths_to_delete)
